// Programacin en C++ para Ingenieros, Ed. Thomson Paraninfo, 2006
// Apndice B: La Biblioteca CDAudioLib

// Fichero AplicacionCDAudio.h
// Especificacin de la clase AplicacionCDAudio

#ifndef _APLICACIONCDAUDIO
#define _APLICACIONCDAUDIO

#include <windows.h>
#include "cdaudio.h"
#include <vector>
#include "resource.h"

class AplicacionCDAudio {

    // Ventana de dilogo de la aplicacin.
    static HWND hDlg;
    // Lista de tracks a reproducir.
    static std::vector<int> vListaTracks;
    // Posicin del track actual en la lista vListaTracks.
    static int iTrackActual;
    // Atributos que mantienen el estado del dispositivo.
    static bool bPausa;
    static bool bResume;
    static bool bPlaying;
    // Atributos que mantienen si las casillas de verificacin
    // Orden Aleatorio y Repetir estn activadas o no.,
    static bool bRandom;
    static bool bRepetir;
  public:
    // Constructor de la clase.
    AplicacionCDAudio(HINSTANCE hinstance);
    // Mtodo que ejecuta la aplicacin.
    int Ejecutar(void);
    // Funcin de atencin a los mensajes de la ventana de dilogo hDlg.
    static BOOL CALLBACK DlgProc(HWND hDlg, UINT msg, WPARAM wParam, 
                                 LPARAM lParam);
    // Funcin de atencin a las interrupciones del dispositivo.
    static void intProcedure(int wParam);
  
  private:
    
    // Mtodo que se ejecuta cuando se pulsa el boton
    // Play/Pause. Cambia el estado de la aplicacin
    // de play a pause y viceversa. Actualiza los
    // botones de la ventana de dilogo hDlg.
    static void botonPlayPausa(void);
    
    // Mtodo que se ejecuta cuando se pulsa el botn Siguiente.
    // Actualiza el track actual al siguiente de la lista de 
    // tracks vListaTracks y si est reproduciendo se incia la 
    // reproduccin del nuevo track.
    static void botonSiguiente(void);
    
    // Mtodo que se ejecuta cuando se pulsa el botn Anterior.
    // Actualiza el track actual al anterior de la lista de 
    // tracks vListaTracks y si est reproduciendo se incia la 
    // reproduccin del nuevo track.
    static void botonAnterior(void);
    
    // Mtodo que se ejecuta cuando se pulsa el botn Expulsar.
    // Abre la puerta del dispositivo, expulsa el CD y actualiza
    // la lista de reproduccin y la ventana de dilogo.
    static void botonExpulsar(void);

    // Mtodo que se ejecuta cuando se pulsa el botn Stop.
    // Si est reproduciendo detiene la reproduccin.
    static void botonStop(void);
    
    // Mtodo que se ejecuta cuando se marca o desmarca la casilla
    // de verificacin Orden aleatorio. Actualiza la lista de 
    // reproduccin vListaTracks segn corresponda.
    static void controlCheckRandom(void);
    
    // Mtodo que se ejecuta cuando se marca o desmarca la casilla
    // de verificacin Repetir. Actualiza el atributo de estado bRepetir
    // que indica si la casilla est marcada o no.
    static void controlCheckRepetir(void);
    
    // Mtodo que genera la lista de tracks a reproducir en un orden
    // aleatorio.
    static void generarListaTracks(void);
    
    // Mtodo que genera la lista de tracks a reproducir en el mismo orden
    // que se encuentran en el CD.
    static void generarListaAleatoriaTracks(void);
    
    // Mtodo que actualiza la lista de reproduccin vListaTracks y la 
    // ventana de dilogo hDlg segn el contenido del CD y las casillas
    // de verificacin.
    static void reset(void);
};

#endif
