// Programacin en C++ para Ingenieros, Ed. Thomson Paraninfo, 2006
// Apndice B: La Biblioteca CDAudioLib

// Fichero cdaudio.h
// API CDAudioLib

#ifndef _CDAUDIO_H
#define _CDAUDIO_H

// Funciones de apertura e inicializacin del CD de Audio.
// Todas estas funciones son sncronas.

   // Funcin para la apertura del dispositivo CD de Audio.
   // Devuelve un valor booleano que indica si se ha
   // producido algn error en el proceso de apertura.
   // Si el dispositivo ya estaba abierto no se produce
   // error. Si no se ejecuta esta funcin antes de utilizar
   // el dispositivo, el comportamiento del resto de funciones
   // no ser correcto.
   bool OpenCDAudio();

// Funciones para el cierre del CD de Audio.
// Todas estas funciones son sncronas.

   // Funcin que para el CD de Audio si est en play
   // y cierra el dispositivo CD de Audio.
   // Devuelve un valor booleano que indica si se ha
   // producido algn error al cerrar el dispositivo.
   bool CloseCDAudio();

// Funciones de consulta del estado del CD de Audio.
// Todas estas funciones son sncronas.

   // Funcin que indica si el dispositivo est abierto.
   bool IsOpen();

   // Funcin que indica si el dispositivo est preparado.
   // Como el cd de audio es un dispositivo de carga automtica
   // siempre devuelve cierto, pero si no se ha abierto el
   // el comportamiento de las funciones no ser correcto.
   bool IsReady();

   // Funcin que indica si el CD de Audio est en modo play.
   bool IsPlaying();

   // Funcin que indica si el CD de Audio est parado.
   bool IsStopped();

   // Funcin que indica si el CD de Audio est en pausa.
   bool IsPaused();

// Funciones de solicitud de Informacin del CD de Audio.
// Todas estas funciones son sncronas.

   // Funcin que informa si hay un CD de Audio
   // insertado en el dispositivo.
   bool EsCDMedia();

   // Funcin que devuelve el nmero de tracks que tiene
   // el CD insertado en el dispositivo.
   // En caso de que no haya un CD de Audio insertado en
   // el dispositivo devuelve un 0.
   int NumeroDeTracks();

   // Funcin que devuelve el nmero de track en el que
   // est posicionada la cabeza lectora del dispositivo.
   // En caso de que no haya un CD de Audio insertado en
   // el dispositivo devuelve un 0.
   int TrackActual();

// Funciones de peticiones de E/S sncronas.
// Todas estas funciones devuelven un booleano
// que indica si se ha producido algn error al
// procesar la peticin de E/S.

   // Funcin que para la reproduccin y resita
   // la cabecera del lector del CD de Audio al
   // principio del primer track del disco.
   bool Stop();

   // Funcin que detiene la reproduccin dejando
   // el cabezal del dispositivo en la posicin
   // que estaba al detenerlo.
   bool Pause();

   // Funcin que reinicia una reproduccin despus de
   // una pausa.
   bool Resume();

   // Funcin que abre la puerta del dispositivo y
   // expulsa el CD.
   bool OpenDoorCD();

   // Funcin que cierra la puerta del dispositivo.
   // No tiene ningn efecto en aquellos dispositivos
   // que no disponen de un mecanismo de cierre
   // de la puerta automtico.
   bool CloseDoorCD();

   // Funcin que reproduce el CD de Audio desde la
   // posicin donde se encuentra situado el cabezal
   // hasta el final del CD. Como es una funcin sincrona
   // la funcin no finaliza hasta que no acaba la reproduccin.
   bool Play();

   // Funcin que reproduce el track correspondiente al
   // numTrack. Si numTrack no est dentro del rango de
   // tracks del dispositivo (entre 1 y NumeroDeTracks())
   // devuelve error. Como es una funcin sncrona la
   // funcin no finaliza hasta que no acaba la reproduccin.
   bool PlayTrack(int numTrack);

   // Funcin que reproduce sucesivamente los tracks comprendidos
   // entre primerTrack y ultimoTrack. Si alguno de los dos
   // parmetros no est dentro del rango de tracks del CD que
   // se encuentra dentro del dispositivo devuelve error.
   // En caso de que los dos parmetros estn dentro del rango
   // pero primerTrack > ultimoTrack no devuelve error pero no
   // reproduce ningn track.
   bool PlayTracksDesdeA(int primerTrack, int ultimoTrack);

// Definicin del tipo de la funcin que se ha de pasar como parmetro
// en una peticin asncrona. Esta funcin se ejecutar en atencin a
// las interrupcin que realice el dispositivo ya sea por finalizacin de
// la peticin, cancelacin, error, etc. El parmetro cdNotificacion se
// corresponde al cdigo de notificacin que indica el motivo de la
// interrupcin.
   typedef void (*CDAUDIOINTPROCEDURE)(int cdNotificacion);

// Constantes para los cdigos de notificacin que, en una
// peticin asncrona, recibe como parmetro la funcin
// que se ha pasado como parmetro en dicha peticin.

   // Constante que indica que la peticin ha finalizado
   // correctamente.
   static const int CDAUDIO_NOTIFY_SUCCESSFUL = 1;

   // Constante que indica que la peticin ha sido substituida
   // por una nueva peticin. Este valor se recibe cuando se
   // realiza una nueva peticin de reproduccin sin que haya
   // finalizado la anterior.
   static const int CDAUDIO_NOTIFY_SUPERSEDED = 2;

   // Constante que indica que la peticin no se ha podido
   // finalizar correctamente. Este valor se recibe cuando se
   // produce algn error al procesar la peticin.
   static const int CDAUDIO_NOTIFY_FAILURE = 3;

   // Constante que indica que la peticin ha sido cancelada.
   // Este valor se recibe cuando se para la reproduccin
   // mediante cualquiera de las funciones de la librera.
   static const int CDAUDIO_NOTIFY_ABORTED = 4;

// Funciones de peticiones de E/S asncronas.
// Todas estas funciones devuelven un booleano
// que indica si se ha producido algn error al
// enviar el comando de E/S y no en el procesado
// del mismo ya que son asncronas. Cuando la peticin
// finalice se ejecutar la funcin intPro que reciben
// estas funciones como parmetro envindole uno de los
// cdigos anteriores que notifica el motivo de la finalizacin

   // Funcin que reproduce el CD de Audio desde la
   // posicin donde se encuentra situado el cabezal
   // hasta el final del CD. Como es una funcin asncrona
   // la funcin finaliza una vez ha comenzado la reproduccin.
   bool Play(CDAUDIOINTPROCEDURE intPro);

   // Funcin que reproduce el track correspondiente al
   // numTrack. Si numTrack no est dentro del rango de
   // tracks del dispositivo (entre 1 y NumeroDeTracks())
   // devuelve error. Como es una funcin asncrona
   // la funcin finaliza una vez ha comenzado la reproduccin.
   bool PlayTrack(int numTrack, CDAUDIOINTPROCEDURE intPro);

   // Funcin que reproduce sucesivamente los tracks comprendidos
   // entre primerTrack y ultimoTrack. Si alguno de los dos
   // parmetros no est dentro del rango de tracks del CD que
   // se encuentra dentro del dispositivo devuelve error.
   // En caso de que los dos parmetros estn dentro del rango
   // pero primerTrack > ultimoTrack no devuelve error pero no
   // reproduce ningn track. Como es una funcin asncrona
   // la funcin finaliza una vez ha comenzado la reproduccin.
   bool PlayTracksDesdeA(int primerTrack, int ultimoTrack,
                         CDAUDIOINTPROCEDURE intPro);

   // Funcin que reinicia una reproduccin despus de
   // una pausa.
   bool Resume(CDAUDIOINTPROCEDURE intPro);

// Funciones que permiten consultar los errores,
// en caso de que se produzcan.

   // Funcin que permite obtener una cadena de caracteres que
   // indica que error se ha producido.
   const char* MensajeError();
#endif
